function [u,v] = LucasKanade(It,rect,It1,u0,v0,rounding)
%It and It1 are grayscale double images in [0,1]
%u0, v0 are optional inputs and denote some tracking model prediction
%rounding is optional; 1 for integer (u,v); 0 for subpixel (u,v)

maxIter = 25;

if nargin < 6
    u0 = 0;
    v0 = 0;
    rounding = 1;
end

if(rounding)
    rect = round(rect);
    u0 = round(u0);
    v0 = round(v0);
end

[numPixY, numPixX] = size(It);

x0 = rect(1);
y0 = rect(2);
xN = rect(3);
yN = rect(4);

colRange0 = x0:xN;
rowRange0 = y0:yN;

numIters = 0;
while(1)
    numIters = numIters + 1;
    if(rounding)
        colRange1 = colRange0 + u0;
        rowRange1 = rowRange0 + v0;
        ItWin = It(rowRange0,colRange0);
        It1Win = It1(rowRange1,colRange1);
    else
        [cols0, rows0] = meshgrid(colRange0,rowRange0);
        cols1 = cols0 + u0;
        rows1 = rows0 + v0;
        ItWin = interp2(It,cols0,rows0);
        It1Win = interp2(It1,cols1,rows1);
    end

    diff = It1Win - ItWin;
    [Ix,Iy] = gradient(It1Win);

    if(0)
        figure(2);clf;
        imshow(ItWin);
        figure(3);clf;
        imshow(It1Win);
        figure(4);clf;
        imshow(diff,[-1,1]);
        figure(5);clf;
        imshow(Ix,[-1,1]);
        figure(6);clf;
        imshow(Iy,[-1,1]);
    end

    diff = diff(:);
    Ix = Ix(:);
    Iy = Iy(:);

%     [U,S,V] = svd([Ix,Iy]);
%     Sinv = S';
%     Sinv(Sinv>1E-10) = 1./Sinv(Sinv>1E-10);
%     uv = V*Sinv*U'*(-diff) + [u0;v0];

    uv = [Ix,Iy]\(-diff) + [u0;v0];
    
    if(rounding)
        uv = round(uv);
    end

    dU = uv(1)-u0;
    dV = uv(2)-v0;
    if(abs(dU) < 0.5 && abs(dV) < 0.5)
        u = uv(1);
        v = uv(2);
        break;
    end
    if(numIters > maxIter)
        u = uv(1);
        v = uv(2);
        disp(['Reached max number of iterations with error = (',num2str(dU),',',num2str(dV),')']);
        break;
    end
    
    u0 = uv(1);
    v0 = uv(2);
end

if(0)
    cols1 = cols0 + u;
    rows1 = rows0 + v;
    ItWin = interp2(It,cols0,rows0);
    It1Win = interp2(It1,cols1,rows1);
    diff = It1Win - ItWin;
    figure(7);clf;
    imshow(diff,[-1,1]);
end

